<?php
/*

Licensed under the The Hornbill Community License: https://wiki.hornbill.com/index.php/The_Hornbill_Community_License_(HCL)

logIncident.php
v1.0.0
10/03/2020

This example script will log a new Incident on a given Hornbill instance, and is designed to be called from a Nagios command.

Configure your request details using the $RequestDetails array entries.

In the following examples, replace yourinstanceid and yourapikey with the relevant CASE SENSITIVE Instance ID and Hornbill API Key that has the required rights to be able to log the Incident.

Example Host Notification command:

	php /opt/hornbill/nagiosscripts/logIncident.php --instance "yourinstanceid" --apikey "yourapikey" --hname="$HOSTNAME$" --notification="$NOTIFICATIONTYPE$" --hstate="$HOSTSTATE$" --houtput="$HOSTOUTPUT$" --heventid="$HOSTEVENTID$" --hproblemid="$HOSTPROBLEMID$"

Example Service Notification command:

	php /opt/hornbill/nagiosscripts/logIncident.php --instance "yourinstanceid" --apikey "yourapikey" --hname="$HOSTNAME$" --notification="$NOTIFICATIONTYPE$" --hstate="$HOSTSTATE$" --houtput="$HOSTOUTPUT$" --heventid="$HOSTEVENTID$" --hproblemid="$HOSTPROBLEMID$" --sdesc="$SERVICEDESC$" --sstate="$SERVICESTATE$" --soutput="$SERVICEOUTPUT$" --seventid="$SERVICEEVENTID$" --sproblemid="$SERVICEPROBLEMID$"

*/
include "espapi_php/esp_method_call.php";
$RequestDetails = array (
	"customerId" => "annab",
	"customerType" => 0,
	"ownerId" => "danielm",
	"teamId" => "InfrastructureSupport",
	"priorityId" => 2,
	"serviceId" => 5,
	"catalogId" => 86,
	"catalogName" => "Server Down",
	"bpmName" =>  "example-hornbill-incident-process"
);

$longopts  = array(
    "instance:", 		//Required
    "apikey:",   		//Required
	"notification::",	//Optional
	"hname::",			//Optional
	"hstate::",			//Optional
	"houtput::",		//Optional
	"heventid::",		//Optional
	"hproblemid::",		//Optional
	"sdesc::",			//Optional
	"sstate::",			//Optional
	"soutput::",		//Optional
	"seventid::",		//Optional
	"sproblemid::",		//Optional
);
$options = getopt("", $longopts);

if (!isset($options["instance"]) || !isset($options["apikey"])) {
	echo "Provide the Instance ID and API Key";
} else {
	$ESP = new EspMethods($options["instance"], $options["apikey"]);
	if ($ESP::$Endpoint == "") {
		echo "no endpoint";
	} else {
		//logIncident if servicestate != OK or hoststate != up AND notificationtype == FLAPPINGSTART or PROBLEM
		if ( (isset($options["sstate"]) && $options["sstate"] != "OK") || (isset($options["hstate"]) && $options["hstate"] != "UP") && ($options["notification"] == "FLAPPINGSTART" && $options["notification"] == "PROBLEM") ) {
			$response = $ESP->logIncident($options, $RequestDetails);
			print_r(join(":",$response));
		} 
	}
}

class EspMethods {
	private static $APIKey;
	public static $Endpoint;

	function __construct($Instance, $Key) {
		self::$APIKey = $Key;
		$url = "https://files.hornbill.com/instances/" . $Instance . "/zoneinfo";
		$httpOptions = array(
			'http' => array(
				'header'  => "Content-type: application/json\r\n",
				'method'  => 'GET'
			)
		);
		$context  = stream_context_create($httpOptions);
		$result = file_get_contents($url, false, $context);
		if ($result === FALSE) { 
			$url = "https://files.hornbill.co/instances/" . $Instance . "/zoneinfo";
			$httpOptions = array(
				'http' => array(
					'header'  => "Content-type: application/json\r\n",
					'method'  => 'GET'
				)
			);
			$context  = stream_context_create($httpOptions);
			$result = file_get_contents($url, false, $context);
			if ($result == FALSE) {
				echo "Could not get instance information. Check the supplied instance ID.";
				return;
			}
		}
		$JSONResponse = json_decode($result);
		self::$Endpoint = $JSONResponse->{'zoneinfo'}->{'endpoint'};
	}
	
	function buildResponse($strMethod, $strService, $strType, $strResponse = "") {
		$arrReturn = array(
			"method" => $strMethod,
			"service" => $strService,
			"type" => $strType,
			"response" => $strResponse
		);
		return $arrReturn;
	}

	function logIncident($cliArgs, $details) {
		$isService = (isset($cliArgs["sdesc"]) && $cliArgs["sdesc"] != "$");
		if (!$isService) {
			$externalRef = $cliArgs["hname"];
			$questionFieldMap = '{"h_custom_a":"'.$cliArgs["hname"].'"}';
			$summary = "NAGIOS HOST ALERT: ".$cliArgs["notification"];
			$description = $summary . PHP_EOL . "Host Name: ".$cliArgs["hname"];
			$description .= PHP_EOL . "Host State: ".$cliArgs["hstate"];
			$description .= PHP_EOL . "Host Output: ".$cliArgs["houtput"];
			$description .= PHP_EOL . "Host Event ID: ".$cliArgs["heventid"];
			$description .= PHP_EOL . "Host Problem ID: ".$cliArgs["hproblemid"];
		} else {
			$externalRef = $cliArgs["hname"] . "::" . $cliArgs["sdesc"];
			$questionFieldMap = '{"h_custom_a":"'.$cliArgs["hname"].'", "h_custom_b":"'.$cliArgs['sdesc'].'"}';
			$summary = "NAGIOS SERVICE ALERT: ".$cliArgs["notification"];
			$description = $summary . PHP_EOL . "Host Name: ".$cliArgs["hname"];
			$description .= PHP_EOL . "Service Description: ".$cliArgs["sdesc"];
			$description .= PHP_EOL . "Service State: ".$cliArgs["sstate"];
			$description .= PHP_EOL . "Service Output: ".$cliArgs["soutput"];
			$description .= PHP_EOL . "Service Event ID: ".$cliArgs["seventid"];
			$description .= PHP_EOL . "Service Problem ID: ".$cliArgs["sproblemid"];
		}
		
		$arrReturn     = array();
		$the_transport = new \Esp\Transport(self::$Endpoint, "xmlmc", "dav");
		$mc            = new \Esp\MethodCall($the_transport);
		$mc->setAPIKey(self::$APIKey);
		$mc->addParam("summary", $summary);
		$mc->addParam("description", $description);
		$mc->addParam("customerId", "{$details["customerId"]}");
		$mc->addParam("customerType", "{$details["customerType"]}");
		$mc->addParam("ownerId", "{$details["ownerId"]}");
		$mc->addParam("teamId", "{$details["teamId"]}");
		$mc->addParam("priorityId", "{$details["priorityId"]}");
		$mc->addParam("serviceId", "{$details["serviceId"]}");
		$mc->addParam("catalogId", "{$details["catalogId"]}");
		$mc->addParam("catalogName", "{$details["catalogName"]}");
		$mc->addParam("externalRefNumber", $externalRef);
		$mc->addParam("bpmName", "{$details["bpmName"]}");
		$mc->addParam("questionFieldMap", $questionFieldMap);

		$mc->invoke("apps/com.hornbill.servicemanager/Incidents", "logIncident");
		if ($mc->getError()) {
			$arrReturn = $this->buildResponse("apps/com.hornbill.servicemanager/Incidents", "logIncident", "ERROR", $mc->getError());
		} else {
			$xmldom    = $mc->getReturnParamsComplexType();
			$nElement  = $xmldom->getElementsByTagName("requestId");
			$arrReturn = $this->buildResponse("apps/com.hornbill.servicemanager/Incidents", "logIncident", "SUCCESS", $nElement->item(0)->textContent);
		}
		return $arrReturn;
	}
}