<?php
/*

Licensed under the The Hornbill Community License: https://wiki.hornbill.com/index.php/The_Hornbill_Community_License_(HCL)

postToWorkspace.php
v1.0.0
10/03/2020

This example script will create a new post within a workspace on a given Hornbill instance, and is designed to be called from a Nagios command.

In the following examples, replace yourinstanceid, yourapikey and yourworkspaceid with the relevant CASE SENSITIVE Instance ID, Hornbill API Key that has the required rights to be able to post to the target workspace, and ID of the target workspace (in the format: urn:buzz:activityStream:9c0e0cb0-4922-4527-a208-3d2411d5d847)

Example Host Notification command:

	php /opt/hornbill/nagiosscripts/postToWorkspace.php --instance "yourinstanceid" --apikey "yourapikey" --workspaceid "yourworkspaceid" --hname="$HOSTNAME$" --notification="$NOTIFICATIONTYPE$" --hstate="$HOSTSTATE$" --houtput="$HOSTOUTPUT$" --heventid="$HOSTEVENTID$" --hproblemid="$HOSTPROBLEMID$"

Example Service Notification command:

	php /opt/hornbill/nagiosscripts/postToWorkspace.php --instance "yourinstanceid" --apikey "yourapikey" --workspaceid "yourworkspaceid" --hname="$HOSTNAME$" --notification="$NOTIFICATIONTYPE$" --hstate="$HOSTSTATE$" --houtput="$HOSTOUTPUT$" --heventid="$HOSTEVENTID$" --hproblemid="$HOSTPROBLEMID$" --sdesc="$SERVICEDESC$" --sstate="$SERVICESTATE$" --soutput="$SERVICEOUTPUT$" --seventid="$SERVICEEVENTID$" --sproblemid="$SERVICEPROBLEMID$"

*/

include "espapi_php/esp_method_call.php";
$longopts  = array(
    "instance:", 		//Required
    "apikey:",   		//Required
	"workspaceid:",		//Required
	"notification::",	//Optional
	"hname::",			//Optional
	"hstate::",			//Optional
	"houtput::",		//Optional
	"heventid::",		//Optional
	"hproblemid::",		//Optional
	"sdesc::",			//Optional
	"sstate::",			//Optional
	"soutput::",		//Optional
	"seventid::",		//Optional
	"sproblemid::",		//Optional
);
$options = getopt("", $longopts);

if (!isset($options["instance"]) || !isset($options["apikey"])) {
	echo "Provide the Instance ID and API Key";
} else {
	$ESP = new EspMethods($options["instance"], $options["apikey"]);
	if ($ESP::$Endpoint == "") {
		echo "no endpoint";
	} else {
		//postToWorkspace
		$response = $ESP->postToWorkspace($options);
		print_r(join(":",$response));
	}
}

class EspMethods {
	private static $APIKey;
	public static $Endpoint;

	function __construct($Instance, $Key) {
		self::$APIKey = $Key;
		$url = "https://files.hornbill.com/instances/" . $Instance . "/zoneinfo";
		$httpOptions = array(
			'http' => array(
				'header'  => "Content-type: application/json\r\n",
				'method'  => 'GET'
			)
		);
		$context  = stream_context_create($httpOptions);
		$result = file_get_contents($url, false, $context);
		if ($result === FALSE) { 
			$url = "https://files.hornbill.co/instances/" . $Instance . "/zoneinfo";
			$httpOptions = array(
				'http' => array(
					'header'  => "Content-type: application/json\r\n",
					'method'  => 'GET'
				)
			);
			$context  = stream_context_create($httpOptions);
			$result = file_get_contents($url, false, $context);
			if ($result == FALSE) {
				echo "Could not get instance information. Check the supplied instance ID.";
				return;
			}
		}
		$JSONResponse = json_decode($result);
		self::$Endpoint = $JSONResponse->{'zoneinfo'}->{'endpoint'};
	}
	
	function buildResponse($strMethod, $strService, $strType, $strResponse = "") {
		$arrReturn = array(
			"method" => $strMethod,
			"service" => $strService,
			"type" => $strType,
			"response" => $strResponse
		);
		return $arrReturn;
	}

	function postToWorkspace($cliArgs) {
		$isService = (isset($cliArgs["sdesc"]) && $cliArgs["sdesc"] != "$");
		if (!$isService) {
			$content = "'''NAGIOS HOST ALERT: ".$cliArgs["notification"]. "'''";
			$content .= PHP_EOL . "Host Name: ".$cliArgs["hname"];
			$content .= PHP_EOL . "Host State: ".$cliArgs["hstate"];
			$content .= PHP_EOL . "Host Output: ".$cliArgs["houtput"];
			$content .= PHP_EOL . "Host Event ID: ".$cliArgs["heventid"];
			$content .= PHP_EOL . "Host Problem ID: ".$cliArgs["hproblemid"];
		} else {
			$content = "'''NAGIOS SERVICE ALERT: ".$cliArgs["notification"]. "'''";
			$content .= PHP_EOL . "Host Name: ".$cliArgs["hname"];
			$content .= PHP_EOL . "Service Description: ".$cliArgs["sdesc"];
			$content .= PHP_EOL . "Service State: ".$cliArgs["sstate"];
			$content .= PHP_EOL . "Service Output: ".$cliArgs["soutput"];
			$content .= PHP_EOL . "Service Event ID: ".$cliArgs["seventid"];
			$content .= PHP_EOL . "Service Problem ID: ".$cliArgs["sproblemid"];
		}
		
		$arrReturn     = array();
		$the_transport = new \Esp\Transport(self::$Endpoint, "xmlmc", "dav");
		$mc            = new \Esp\MethodCall($the_transport);
		$mc->setAPIKey(self::$APIKey);
		$mc->addParam("activityStreamID", $cliArgs["workspaceid"]);
		$mc->addParam("content", $content);
		$mc->addParam("visibility", "members");
		$mc->invoke("activity", "activityPostMessage");
		if ($mc->getError()) {
			$arrReturn = $this->buildResponse("activity", "activityPostMessage", "ERROR", $mc->getError());
		} else {
			$xmldom    = $mc->getReturnParamsComplexType();
			$nElement  = $xmldom->getElementsByTagName("activityID");
			$arrReturn = $this->buildResponse("activity", "activityPostMessage", "SUCCESS", $nElement->item(0)->textContent);
		}
		return $arrReturn;
	}
}